(*  Modulo de implementacion de una agenda doblemente enlazada-Tipo OPACO *)

IMPLEMENTATION MODULE Agenda;

FROM Storage IMPORT ALLOCATE, DEALLOCATE, Available;
FROM MiStr IMPORT Comparar, Copiar, CADENA;
FROM IO IMPORT WrLn, WrStr;

TYPE AGENDA=POINTER TO CABECERA;
   LISTIN=POINTER TO REGISTRO;

   CABECERA=RECORD
                 primero:LISTIN;
                 actual:LISTIN;
            END;

   REGISTRO=RECORD
                 nombre, telefono:CADENA;
                 ant, sig:LISTIN;
            END;

PROCEDURE Crear(VAR error:BOOLEAN):AGENDA;
          (* Devuelve un puntero a una cabecera de agenda. *)
VAR
  a:AGENDA;
BEGIN
    error:=FALSE;
       (* Uso de Available *)
    IF Available(SIZE(CABECERA)) THEN
       NEW(a);
       a^.primero:=NIL;
       a^.actual:=NIL;
    ELSE
       error:=TRUE;
       a:=NIL;
    END;
    RETURN a;
END  Crear;


   (* Procedimiento local al modulo *)
PROCEDURE EsVacia(agenda:AGENDA):BOOLEAN;
BEGIN
    RETURN agenda^.primero=NIL;
END EsVacia;


   (* Procedimiento local al modulo *)
PROCEDURE Esta(agenda:AGENDA; nombre:ARRAY OF CHAR):BOOLEAN;
          (* Me devuelve TRUE si un nombre dado esta en la agenda *)
VAR aux:LISTIN;
    esta:BOOLEAN;
BEGIN
  esta:=FALSE;
  IF NOT (agenda=NIL) AND NOT EsVacia(agenda) THEN
      aux:=agenda^.actual;
      IF Comparar(aux^.nombre,nombre)=-1 THEN
         WHILE (aux<>NIL) AND (Comparar(aux^.nombre,nombre)=-1) DO
           aux:=aux^.sig;
         END;(*WHILE*)
      ELSE
         WHILE (aux<>NIL) AND (Comparar(aux^.nombre,nombre)=1) DO
           aux:=aux^.ant;
         END;(*WHILE*)
      END;(* IF*)
      IF (aux<>NIL) AND (Comparar(aux^.nombre,nombre)=0) THEN
         esta:=TRUE
      END;
  END;
  RETURN esta;
END Esta;


PROCEDURE Insertar(VAR agenda:AGENDA; nombre, telefono:ARRAY OF CHAR; VAR error:BOOLEAN);
          (* Inserta en la agenda, si no es posible( no hay memoria disponible)
             => error=TRUE*)
VAR
  aux,nuevo:LISTIN;
BEGIN
  error:=FALSE;
  IF (agenda=NIL) OR (NOT(Available(SIZE(REGISTRO)))) THEN
     error:=TRUE;
       (*Si no esta creada la agenda o no hay memoria
         disponible para insertar *)
  ELSIF NOT Esta(agenda,nombre) THEN
    NEW(nuevo);
    Copiar(nuevo^.nombre,nombre);         (* Permito la asignacion tambien *)
    Copiar(nuevo^.telefono,telefono);
    nuevo^.ant:=NIL;
    nuevo^.sig:=NIL;
    IF EsVacia(agenda) THEN
       agenda^.primero:=nuevo;
       agenda^.actual:=nuevo;
    ELSIF NOT Esta(agenda,nombre) THEN
        (* solo inserta un elemento si no esta *)
       aux:=agenda^.actual;
       IF Comparar(aux^.nombre,nombre)=-1 THEN
          WHILE (aux^.sig<>NIL) AND (Comparar(aux^.nombre,nombre)=-1) DO
            aux:=aux^.sig;
          END;(*WHILE*)
          IF (aux^.sig=NIL) AND (Comparar(aux^.nombre,nombre)=-1) THEN
               nuevo^.ant:=aux;
               aux^.sig:=nuevo
          ELSE
             nuevo^.sig:=aux;
             nuevo^.ant:=aux^.ant;
             aux^.ant^.sig:=nuevo;
             aux^.ant:=nuevo
          END;
       ELSE
          WHILE (aux^.ant<>NIL) AND (Comparar(aux^.nombre,nombre)=1) DO
            aux:=aux^.ant;
          END;(*WHILE*)
          IF (aux^.ant=NIL) AND (Comparar(aux^.nombre,nombre)=1) THEN
               nuevo^.sig:=aux;
               aux^.ant:=nuevo;
               agenda^.primero:=nuevo;
          ELSE
             nuevo^.ant:=aux;
             nuevo^.sig:=aux^.sig;
             aux^.sig^.ant:=nuevo;
             aux^.sig:=nuevo
          END;
       END;(*IF Comparar*)
       agenda^.actual:=nuevo;
    END;(*IF Vacia*)
  END;
END Insertar;


(* Procedimiento local al modulo *)
PROCEDURE Visualizar(listin:LISTIN);
          (* Visualiza un nombre y un telefono*)
BEGIN
    WrLn;
    WrStr("-----------------------------------------------------------");
    WrLn;
    WrStr(" Nombre: ");WrStr(listin^.nombre);
    WrStr(". Telefono: ");WrStr(listin^.telefono);
END Visualizar;


PROCEDURE Consultar(agenda:AGENDA; nombre:ARRAY OF CHAR; VAR error:BOOLEAN);
          (* Visualiza el telefono que corresponda al nombre que le demos,
             si no existe este o la estructura=> error=TRUE *)
VAR aux:LISTIN
;BEGIN
    error:=FALSE;
    IF (agenda=NIL) OR EsVacia(agenda) OR NOT(Esta(agenda,nombre)) THEN
       error:=TRUE; (* La agenda no esta creada o esta vacia o
                       el nombre no esta en la agenda*)
    ELSE       (* Sabemos que el elemento esta *)
      aux:=agenda^.actual;
      IF Comparar(aux^.nombre,nombre)=1 THEN
         WHILE Comparar(aux^.nombre,nombre)=1 DO
           aux:=aux^.ant;
         END;(*WHILE*)
      ELSE
         WHILE Comparar(aux^.nombre,nombre)=-1 DO
           aux:=aux^.sig;
         END;(*WHILE*)
      END;
      Visualizar(aux);
      agenda^.actual:=aux
   END;(* IF*)
END Consultar;


PROCEDURE Listar(agenda:AGENDA; VAR error:BOOLEAN);
          (* Lista la agenda, si no fu creada => error:=TRUE*)
VAR aux:LISTIN;
BEGIN
    error:=FALSE;
    IF (agenda=NIL) THEN  (* No hay agenda que listar *)
       error:=TRUE;
    ELSE
       aux:=agenda^.primero;
       WHILE aux<>NIL DO
             Visualizar(aux);
             aux:=aux^.sig;
       END;
    END;
END Listar;


PROCEDURE Modificar(VAR agenda:AGENDA; nombre, telefono:ARRAY OF CHAR; VAR error:BOOLEAN);
          (* Cambia el telefono correspondiente al nombre que le demos, si
             no existe este o la estructura=> error=TRUE*)
VAR aux:LISTIN;
BEGIN
    error:=FALSE;
    IF (agenda=NIL) OR EsVacia(agenda) OR NOT(Esta(agenda,nombre)) THEN
       error:=TRUE;
    ELSE                       (* El registro esta *)
      aux:=agenda^.actual;
      IF Comparar(aux^.nombre,nombre)=1 THEN
         WHILE Comparar(aux^.nombre,nombre)=1 DO
           aux:=aux^.ant;
         END;(*WHILE*)
      ELSE
         WHILE Comparar(aux^.nombre,nombre)=-1 DO
           aux:=aux^.sig;
         END;(*WHILE*)
      END;
      agenda^.actual:=aux;
      Copiar(agenda^.actual^.telefono,telefono);
    END;
END Modificar;

PROCEDURE Borrar(VAR agenda:AGENDA; nombre:ARRAY OF CHAR; VAR error:BOOLEAN);
          (* Borra un nombre y su telefono de la agenda, si no existe este
             o la estructura =>error=TRUE*)
VAR
  aux:LISTIN;
BEGIN
    error:=FALSE;
    IF (agenda=NIL) OR EsVacia(agenda) OR NOT(Esta(agenda,nombre)) THEN
       error:=TRUE;
    ELSE
      aux:=agenda^.actual;      (* Busco el elemento *)
      IF Comparar(aux^.nombre,nombre)=1 THEN
         WHILE Comparar(aux^.nombre,nombre)=1 DO
           aux:=aux^.ant;
         END;(*WHILE*)
      ELSE
         WHILE Comparar(aux^.nombre,nombre)=-1 DO
           aux:=aux^.sig;
         END;(*WHILE*)
      END;
      (* aux contiene el elemento a borrar *)
      IF aux^.ant=NIL THEN  (* es el primer elemento *)
          agenda^.primero:=aux^.sig;
          agenda^.actual:=agenda^.primero
      ELSE
         aux^.ant^.sig:=aux^.sig;
         IF aux^.sig<>NIL THEN  (*No es el ultimo *)
            aux^.sig^.ant:=aux^.ant
         END;
         agenda^.actual:=aux^.ant
      END;
      DISPOSE(aux);
    END;
END Borrar;


PROCEDURE BorrarAgenda(VAR agenda:AGENDA;VAR error:BOOLEAN);
BEGIN
    IF agenda=NIL THEN
       error := TRUE   (* No habia agenda para borrar *)
    ELSE
       error:=FALSE;
       WHILE NOT EsVacia(agenda) DO
         agenda^.actual:=agenda^.primero; (* actual apunta al primero*)
         agenda^.primero:=agenda^.primero^.sig; (* Primero apunta al segundo elemento*)
         DISPOSE(agenda^.actual);
       END;
    END;
END BorrarAgenda;

PROCEDURE Destruir(VAR agenda:AGENDA;VAR error:BOOLEAN);
BEGIN
    BorrarAgenda(agenda,error);
    IF NOT error THEN
       DISPOSE(agenda);
    END;
END Destruir;

END Agenda.








